part of katana_module;

/// Class for handling reroute definitions.
///
/// Set [value] to configure the route settings.
@immutable
abstract class RerouteConfig implements ModuleHook {
  const RerouteConfig();

  /// Reroute settings that are tied to.
  ///
  /// If [value] is `true`, the route will be changed to the path of [key].
  Map<String, bool Function(BuildContext context)> get value;

  static RerouteConfig? _fromMap(DynamicMap map) {
    if (!map.containsKey("type")) {
      return null;
    }
    switch (map.get("type", "")) {
      case LoginRequiredRerouteConfig._type:
        return LoginRequiredRerouteConfig._fromMap(map);
      case RegistrationRequiredRerouteConfig._type:
        return RegistrationRequiredRerouteConfig._fromMap(map);
      case UserDocumentQueryRerouteConfig._type:
        return UserDocumentQueryRerouteConfig._fromMap(map);
    }
  }

  /// Convert the reroute config to [DynamicMap].
  DynamicMap toMap();

  /// Run it the first time the app is launched.
  @override
  @mustCallSuper
  Future<void> onInit(BuildContext context) => Future.value();

  /// Runs when restoring authentication.
  @override
  @mustCallSuper
  Future<void> onRestoreAuth(BuildContext context) => Future.value();

  /// Runs after authentication has taken place.
  ///
  /// It is also called after registration or login has been completed.
  @override
  @mustCallSuper
  Future<void> onAfterAuth(BuildContext context) => Future.value();

  /// It is executed after the boot process is finished and
  /// before transitioning to another page.
  @override
  @mustCallSuper
  Future<void> onBeforeFinishBoot(BuildContext context) => Future.value();
}
